//////////////////////////////////////////////////////////////////////////
//
// Hall A analyzer replay script
//
// if there is no rootlogon.C, execute following command in analyzer first
// gSystem->Load("<your lib name>.so");
// gSystem->AddIncludePath("-I<your include path>");
// gInterpreter->AddIncludePath("<your include path>");
//
//////////////////////////////////////////////////////////////////////////
//	
//	Author : Jin Huang (jinhuang@jlab.org)    Aug 2007
//
//	Modify History:
//      Mar 2008    Jin Huang
//          add physics module support
//      Mar 2008    Jin Huang
//          separate two modes: physics replay and detector replay
//          also make it smarter
//
//////////////////////////////////////////////////////////////////////////

#include "def.h"

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <string>
#include <string.h>

#include "TSystem.h"
#include "TROOT.h"
#include "THaEvent.h"
#include "THaRun.h"
#include "THaAnalyzer.h"
#include "THaTriggerPlane.h"
#include "THaScintPlane.h"

#include "THaBigBiteTpMwdc.h"
#include "THaTriggerPlane.h"
#include "THaScintPlane.h"
#include "MWDC.h"
#include "THaOpticsAnalytical.h"

#include "THaHRS.h"
#include "THaPrimaryKine.h"
#include "THaSecondaryKine.h"
#include "THaIdealBeam.h"

using namespace std;


typedef enum 
{
    kReplayBigBiteTp    =1<<0,
    kReplayBigBiteMWDC  =1<<1,
    kReplayBigBiteOther =1<<2,
    kReplayHRS          =1<<4,
    kReplayBeam         =1<<5,
    kReplayAllDet=kReplayBigBiteTp+kReplayBigBiteMWDC
    +kReplayBigBiteOther+kReplayHRS+kReplayBeam,
    kReplayPhysics      =1<<6,
    kReplayScalers      =1<<7
}eReplayMode;

void replay(Int_t runnumber=0,Int_t all=0, 
            Int_t ReplayMode=kReplayBigBiteTp,
            Int_t DefReplayNum=-1,
            char* OutFileFormat=OUT_FILE_FORMAT
            )
{
    //general replay script
    //for detector or physics replay, please use replay_det or physics reply 

    cout<<"replay: Init analyzer ..."<<endl;
    THaAnalyzer* analyzer = THaAnalyzer::GetInstance();
    if( analyzer ) {
        analyzer->Close();
    } else {
        analyzer = new THaAnalyzer;
    }



    // step 1: add detectors
    THaBigBite* pB=0;
    if (ReplayMode&(kReplayBigBiteTp+kReplayBigBiteMWDC+kReplayBigBiteOther))
    {
        cout<<"replay: Adding BigBite ..."<<endl;
        pB=new THaBigBite("BB","BigBite"); 
        gHaApps->Add(pB);
    }
    if (ReplayMode&(kReplayBigBiteTp))
        pB->AddDetector(new THaTriggerPlane("tp","Trigger Plane",pB));

    if (ReplayMode&(kReplayBigBiteMWDC))
        pB->AddDetector(new TreeSearch::MWDC("mwdc","MWDC",pB));

    if (ReplayMode&(kReplayBigBiteOther))
        pB->MountOptics(new THaOpticsAnalytical("optics","BigBite Optics Child Class",pB));

    if (ReplayMode&(kReplayBeam))
    {
        cout<<"replay: Adding Beam ..."<<endl;
        gHaApps->Add(new THaIdealBeam("Beam","Idea Beam, for Test Only"));
    }

    if (ReplayMode&(kReplayHRS))
    {
        cout<<"replay: Adding HRS ..."<<endl;
        gHaApps->Add(new THaHRS("L","Left HRS"));
    }

    //Step 1.5: load Physics modules
    if (ReplayMode&(kReplayPhysics))
    {
        cout<<"replay: adding Physics modules ..."<<endl;

        THaPrimaryKine *PriKine=new THaPrimaryKine(
            "PriKine","kinematics of scattering of the primary (beam) particle",
            "L","Beam",.938272029);
        gHaApps->Add(PriKine);

        THaSecondaryKine *SecKine=new THaSecondaryKine(
            "PriKine","kinematics of scattering of the primary (beam) particle",
            "BB","PriKine",.938272029);
        gHaApps->Add(SecKine);

        // Enable scalers
        analyzer->EnableScalers();
    }




    // step 2: setup run information

    int nrun, nev;
    int found = 0;
    const char** path = 0;
    char filename[300],buf[300];  
    FILE *fd;

    while( found==0 ) {
        if (runnumber<=0)
        {
            cout << "\nreplay: Please enter a Run Number (-1 to exit):";
            cin >> nrun;
            if( nrun<=0 ) break;
        }
        else 
            nrun=runnumber;

        path=PATHS;

        while ( path && *path ) {
            sprintf(filename,RAW_DATA_FORMAT,*path,nrun);
            cout<<"replay: Try file "<<filename<<endl;
            fd = fopen(filename,"r");
            if (fd != NULL) {
                found = 1;
                cout << "replay: Will analyze file "<<filename<<endl;
                fclose(fd);
                break;
            }
            path++;
        }
        if ( (!path || !*path) && !found ) {
            cout << "replay: File not found. \n";
            if (runnumber>0) runnumber=0;
        }
    }
    if(nrun<=0) return;

    if (all==0)
    {
        cout << "\nreplay: Number of Events to analyze "
            <<" (default=";
        if (DefReplayNum>0)
            cout<<DefReplayNum<<"):";
        else
            cout<<"replay all):";

        //trick to supports blank inputs
        fgets(buf,300,stdin);
        if (sscanf(buf,"%d\n",&nev)<1)
        {
            fgets(buf,300,stdin);
            if (sscanf(buf,"%d\n",&nev)<1)
                nev=DefReplayNum;   
        }
    }
    else 
        nev=all;

    cout<<"replay: Setup run inputs/outputs ..."<<endl;

    THaRun run(filename);
    run.SetFirstEvent( 1 );
    if(nev>-1) run.SetLastEvent(nev);

    char sumname[300];
    char outname[300];
    sprintf(outname,OutFileFormat,nrun);
    analyzer->SetOutFile( outname );
    analyzer->SetOdefFile(OUT_DEFINE_FILE);
    analyzer->SetCutFile(CUT_DEFINE_FILE);  
    sprintf(sumname,SUMMARY_PHYSICS_FORMAT,nrun);
    analyzer->SetSummaryFile(sumname); // optional
    analyzer->SetEvent( new THaEvent );



    // step 3: run it
    cout<<"replay: Start Process ..."<<endl;
    analyzer->Process(run);     // start the actual analysis



    // step 3: clean up
    run.Close();
    gHaApps->Delete();
    analyzer->Close();

    cout<<"replay: YOU JUST ANALYZED RUN number "<<nrun<<"."<<endl;
}

/////////////////////////////////////////////////////////////////

void replay_det(Int_t runnumber=0,Int_t all=0)
{
    //detector reply 

    replay(runnumber,all,
        kReplayAllDet,
        1000,
        OUT_FILE_FORMAT_DET
        );
}

/////////////////////////////////////////////////////////////////
void replay_phys(Int_t runnumber=0,Int_t all=0)
{
    //physics reply 

    replay(runnumber,all,
        kReplayAllDet+kReplayPhysics+kReplayScalers,
        -1,
        OUT_FILE_FORMAT_PHYS
        );
}

